@@ -54,7 +54,7 @@ pub use err::{Error, NewClientSeedError, NewSkySparkClientError};
 pub use grid::{Grid, ParseJsonGridError};
 pub use hs_types::{Date, DateTime, Time};
 pub use raystack_core::Coord;
-pub use raystack_core::Number;
+pub use raystack_core::{BasicNumber, Number, ScientificNumber};
 pub use raystack_core::{is_tag_name, ParseTagNameError, TagName};
 pub use raystack_core::{Marker, Na, RemoveMarker, Symbol, Uri, Xstr};
 pub use raystack_core::{ParseRefError, Ref};
